from checkov.terraform.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_check import BaseResourceCheck


class SNSTopicEncryption(BaseResourceCheck):
    def __init__(self):
        name = "Ensure all data stored in the SNS topic is encrypted"
        id = "CKV_AWS_26"
        supported_resources = ['aws_sns_topic']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for encryption configuration at aws_sns_topic:
            https://www.terraform.io/docs/providers/aws/r/sns_topic.html
        :param conf: aws_s3_bucket configuration
        :return: <CheckResult>
        """
        if 'kms_master_key_id' in conf.keys():
            if conf['kms_master_key_id']:
                return CheckResult.SUCCESS
        return CheckResult.FAILURE


check = SNSTopicEncryption()



import logging
from checkov.terraform.models.enums import ContextCategories

supported_definitions = [ContextCategories.RESOURCE.name]


class ParserRegistry():
    context_parsers = {}
    definitions_context = {}

    def __init__(self):
        self.logger = logging.getLogger(__name__)

    def register(self, parser):
        self.context_parsers[parser.definition_type] = parser

    def enrich_definitions_context(self, definitions):
        (tf_file, definition_blocks_types) = definitions
        definition_blocks_types = {x.upper(): definition_blocks_types[x] for x in definition_blocks_types.keys()}
        for definition_type in definition_blocks_types.keys():
            if definition_type in supported_definitions:
                context_parser = self.context_parsers[definition_type]
                definition_blocks = definition_blocks_types[definition_type]
                self.definitions_context[tf_file] = context_parser.run(tf_file, definition_blocks)

        return self.definitions_context


parser_registry = ParserRegistry()



import logging


class Registry:
    checks = {}

    def __init__(self):
        self.logger = logging.getLogger(__name__)

    def register(self, check):
        for resource in check.supported_resources:
            if resource not in self.checks.keys():
                self.checks[resource] = []
            self.checks[resource].append(check)

    def get_checks(self, resource):
        if resource in self.checks.keys():
            return self.checks[resource]
        return []

    def scan(self, block, scanned_file, skipped_checks):
        resource = list(block.keys())[0]
        resource_conf = block[resource]
        results = {}
        checks = self.get_checks(resource)
        for check in checks:
            skip = False
            if skipped_checks:
                skip = check.id in skipped_checks
            resource_name = list(resource_conf.keys())[0]
            resource_conf_def = resource_conf[resource_name]
            self.logger.debug("Running check: {} on file {}".format(check.name, scanned_file))
            result = check.run(scanned_file=scanned_file, resource_configuration=resource_conf_def,
                               resource_name=resource_name, resource_type=resource, skip=skip)
            results[check] = result
        return results


resource_registry = Registry()



from enum import Enum


class CheckResult(str, Enum):
    SUCCESS = "SUCCESS"
    FAILURE = "FAILURE"
    UNKNOWN = "UNKNOWN"
    SKIPPED = "SKIPPED"


class CheckCategories(Enum):
    LOGGING = 1
    ENCRYPTION = 2
    GENERAL_SECURITY = 3
    NETWORKING = 4
    IAM = 5
    BACKUP_AND_RECOVERY = 6


class OutputFormat(Enum):
    CONSOLE = 1
    JSON = 2
    JUNIT_XML = 3


class ContextCategories(Enum):
    PROVIDER = 1
    TERRAFORM = 2
    LOCALS = 3
    MODULE = 4
    DATA = 5
    RESOURCE = 6
    VARIABLE = 7
    OUTPUT = 8



from checkov.terraform.context_parsers.base_parser import BaseContextParser


class ResourceContextParser(BaseContextParser):
    def __init__(self):
        definition_type = 'RESOURCE'
        super().__init__(definition_type=definition_type)

    def enrich_definition_block(self, block):
        """
        Enrich the context of a Terraform resource block
        :param block: Terraform resource block, key-value dictionary
        :return: Enriched resource block context
        """
        parsed_file_lines = self._filter_file_lines()

        for i, resource_block in enumerate(block):
            resource_type = next(iter(resource_block.keys()))
            resource_name = next(iter(resource_block[resource_type]))
            if not self.context.get(resource_type):
                self.context[resource_type] = {}
            if not self.context.get(resource_type).get(resource_name):
                self.context[resource_type][resource_name] = {}
            for line_num, line in parsed_file_lines:
                line_tokens = [x.replace('"', "") for x in line.split()]
                if all(x in line_tokens for x in ['resource', resource_type, resource_name]):
                    start_line = line_num
                    end_line = self._compute_definition_end_line(line_num)
                    self.context[resource_type][resource_name]["start_line"] = start_line
                    self.context[resource_type][resource_name]["end_line"] = end_line
                    self.context[resource_type][resource_name]["code_lines"] = self.file_lines[start_line - 1: end_line]
        return self.context


parser = ResourceContextParser()



from checkov.terraform.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_check import BaseResourceCheck


class RDSPubliclyAccessible(BaseResourceCheck):
    def __init__(self):
        name = "Ensure all data stored in the RDS bucket is not public accessible"
        id = "CKV_AWS_17"
        supported_resources = ['aws_db_instance','aws_rds_cluster_instance']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for publicly_accessible configuration at aws_db_instance:
            https://www.terraform.io/docs/providers/aws/d/db_instance.html
        :param conf: publicly_accessible configuration
        :return: <CheckResult>
        """
        if 'publicly_accessible' in conf.keys():
            key = conf['publicly_accessible'][0]
            if key:
                return CheckResult.FAILURE
        return CheckResult.SUCCESS


check = RDSPubliclyAccessible()



from checkov.terraform.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_check import BaseResourceCheck


class S3Encryption(BaseResourceCheck):
    def __init__(self):
        name = "Ensure all data stored in the S3 bucket is securely encrypted at rest"
        id = "CKV_AWS_19"
        supported_resources = ['aws_s3_bucket']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for encryption configuration at aws_s3_bucket:
            https://www.terraform.io/docs/providers/aws/r/s3_bucket.html
        :param conf: aws_s3_bucket configuration
        :return: <CheckResult>
        """
        if 'server_side_encryption_configuration' in conf.keys():
            sse_block = conf['server_side_encryption_configuration']
            if 'rule' in sse_block[0].keys():
                rule_block = sse_block[0]['rule']
                if 'apply_server_side_encryption_by_default' in rule_block[0].keys():
                    encryption_block = rule_block[0]['apply_server_side_encryption_by_default']
                    if  'sse_algorithm' in encryption_block[0].keys():
                        return CheckResult.SUCCESS
        return CheckResult.FAILURE


check = S3Encryption()



from checkov.terraform.checks.resource.base_check import BaseResourceCheck
from checkov.terraform.models.enums import CheckResult, CheckCategories


class GKEClusterLogging(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Stackdriver Logging is set to Enabled on Kubernetes Engine Clusters"
        id = "CKV_GCP_1"
        supported_resources = ['google_container_cluster']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for password configuration at azure_instance:
            https://www.terraform.io/docs/providers/google/r/compute_ssl_policy.html
        :param conf: google_compute_ssl_policy configuration
        :return: <CheckResult>
        """
        if 'logging_service' in conf.keys():
            if conf['logging_service'][0] == "none":
                return CheckResult.FAILURE
        return CheckResult.SUCCESS


check = GKEClusterLogging()



from termcolor import colored
from checkov.terraform.models.enums import CheckResult
from colorama import init, Fore

init(autoreset=True)


class Record():
    check_id = ""
    check_name = ""
    check_result = None
    check_class = ""
    code_block = ""
    file_path = ""
    file_line_range = []
    resource = ""

    def __init__(self, check_id, check_name, check_result, code_block, file_path, file_line_range, resource,
                 check_class):
        self.check_id = check_id
        self.check_name = check_name
        self.check_result = check_result
        self.code_block = code_block
        self.file_path = file_path
        self.file_line_range = file_line_range
        self.resource = resource
        self.check_class = check_class

    @staticmethod
    def code_line_string(line_num, line):
        return "\t\t" + Fore.BLUE + str(line_num) + Fore.WHITE + ' | ' + Fore.YELLOW + line

    def __str__(self):
        if self.check_result == CheckResult.SUCCESS:
            status = "Passed"
            status_color = "green"
        else:
            status = "Failed"
            status_color = "red"
        check_message = colored("Check: \"{}\"\n".format(self.check_name), "white")
        file_details = colored("{}:{}\n\n".format(self.file_path, "-".join([str(x) for x in self.file_line_range])),
                               "magenta")
        code_lines = "{}\n".format("".join([self.code_line_string(*code_line) for code_line in self.code_block]))
        status_message = colored("\t{} for resource: {}\n".format(status, self.resource), status_color)
        return check_message + file_details + code_lines + status_message
